'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.storage = exports.metadata = exports.auth = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _jfs = require('jfs');

var _jfs2 = _interopRequireDefault(_jfs);

var _SensorAttribute = require('../SensorAttribute');

var _SensorAttribute2 = _interopRequireDefault(_SensorAttribute);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var tokens,
    meta,
    stores = {};

var auth = exports.auth = function auth(req, config, data, cb) {
	if (!tokens) tokens = new _jfs2.default(config.file);

	tokens.get(req.auth.token, function (err, user) {
		if (err || !user) return cb('Unauthorized token.');
		cb(null, user);
	});
};

var metadata = exports.metadata = function metadata(req, config, data, cb) {
	if (!meta) meta = new _jfs2.default(config.file);

	meta.get(data.getDeviceId(), function (err, meta) {
		if (err || !meta) return cb('No metadata for ' + data.getDeviceId() + '.');

		var attributes = meta.map(function (info) {
			var attr = new _SensorAttribute2.default(info.name);

			if (!Array.isArray(info.converters)) {
				return cb('No converters for ' + data.getDeviceId() + '.');
			}

			info.converters.forEach(function (conv) {
				var type = conv,
				    value;
				if (Array.isArray(conv)) {
					;

					var _conv = _slicedToArray(conv, 2);

					type = _conv[0];
					value = _conv[1];
				}attr.addConverter(type, value);
			});

			if (Array.isArray(info.calibrators)) {
				info.calibrators.forEach(function (fn) {
					return attr.addCalibrator(new Function(fn));
				});
			}

			if (Array.isArray(info.validators)) {
				info.validators.forEach(function (val) {
					var type = val,
					    value;
					if (Array.isArray(val)) {
						;

						var _val = _slicedToArray(val, 2);

						type = _val[0];
						value = _val[1];
					}attr.addValidator(type, value);
				});
			}

			return attr;
		});

		data.setAttributes(attributes);
		cb();
	});
};

var storage = exports.storage = function storage(req, config, data, cb) {
	var deviceId = data.getDeviceId();
	var store = stores[deviceId];

	if (!store) store = stores[deviceId] = new _jfs2.default(config.path + ('/store.' + deviceId + '.json'));

	store.save(data.getData(), cb);
};