'use strict';

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _connect = require('connect');

var _connect2 = _interopRequireDefault(_connect);

var _bunyan = require('bunyan');

var _bunyan2 = _interopRequireDefault(_bunyan);

var _verifyRequest = require('./middleware/verifyRequest');

var _verifyRequest2 = _interopRequireDefault(_verifyRequest);

var _parsePayload = require('./middleware/parsePayload');

var _parsePayload2 = _interopRequireDefault(_parsePayload);

var _createSensorData = require('./middleware/createSensorData');

var _createSensorData2 = _interopRequireDefault(_createSensorData);

var _authenticate = require('./middleware/authenticate');

var _authenticate2 = _interopRequireDefault(_authenticate);

var _setMetadata = require('./middleware/setMetadata');

var _setMetadata2 = _interopRequireDefault(_setMetadata);

var _concava = require('./middleware/concava');

var _debug = require('./middleware/debug');

var _debug2 = _interopRequireDefault(_debug);

var _store = require('./middleware/store');

var _store2 = _interopRequireDefault(_store);

var _errorHandler = require('./middleware/errorHandler');

var _errorHandler2 = _interopRequireDefault(_errorHandler);

var _config = require('../config.js');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var app = (0, _connect2.default)();

// Configuration


var port = parseInt(_config2.default.port) || 3000;

// Logger
var logFile = _config2.default.logFile || '/tmp/output.log';
var log = _bunyan2.default.createLogger({
	name: _config2.default.logName || 'concava',
	streams: [{ level: 'warn', stream: process.stdout }, { level: _config2.default.debug ? 'debug' : 'info', path: logFile }]
});

// Exception handling
process.on('uncaughtException', function (err) {
	log.error({ type: 'uncaught-exception', stack: err.stack }, '' + err);
});

// Add timestamp and logger to request
app.use(function (req, res, next) {
	req.start = new Date();
	req.log = log;
	next();
});

app.use((0, _verifyRequest2.default)());
app.use((0, _parsePayload2.default)());
app.use((0, _createSensorData2.default)());
app.use((0, _authenticate2.default)(_config2.default.auth || {}));
app.use((0, _setMetadata2.default)(_config2.default.metadata));
app.use((0, _concava.convert)());
app.use((0, _concava.calibrate)());
app.use((0, _concava.validate)());
app.use((0, _debug2.default)(_config2.default.debug));
app.use((0, _store2.default)(_config2.default.storage));
app.use((0, _errorHandler2.default)(_config2.default.auth.byToken));

// End request
app.use(function (req, res, next) {
	res.end();
	next();
});

// Start server
_http2.default.createServer(app).listen(port);
log.info('Listening on port ' + port + '.');