'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _connect = require('connect');

var _connect2 = _interopRequireDefault(_connect);

var _bunyan = require('bunyan');

var _bunyan2 = _interopRequireDefault(_bunyan);

var _verifyRequest = require('./middleware/verifyRequest');

var _verifyRequest2 = _interopRequireDefault(_verifyRequest);

var _parsePayload = require('./middleware/parsePayload');

var _parsePayload2 = _interopRequireDefault(_parsePayload);

var _createSensorData = require('./middleware/createSensorData');

var _createSensorData2 = _interopRequireDefault(_createSensorData);

var _authenticate = require('./middleware/authenticate');

var _authenticate2 = _interopRequireDefault(_authenticate);

var _setMetadata = require('./middleware/setMetadata');

var _setMetadata2 = _interopRequireDefault(_setMetadata);

var _concava = require('./middleware/concava');

var _debug = require('./middleware/debug');

var _debug2 = _interopRequireDefault(_debug);

var _store = require('./middleware/store');

var _store2 = _interopRequireDefault(_store);

var _errorHandler = require('./middleware/errorHandler');

var _errorHandler2 = _interopRequireDefault(_errorHandler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Server = function () {
	function Server() {
		_classCallCheck(this, Server);

		this._debug = 0;
		this._logFile = '/tmp/output.log';
		this._logName = 'concava';
		this._port = 3000;
		this._payloadMaxSize = '512kb';
	}

	_createClass(Server, [{
		key: 'setDebugMode',
		value: function setDebugMode(debug) {
			this._debug = debug ? 1 : 0;
			return this;
		}
	}, {
		key: 'getDebugMode',
		value: function getDebugMode() {
			return this._debug;
		}
	}, {
		key: 'setLogFile',
		value: function setLogFile(file) {
			this._logFile = '' + file;
			return this;
		}
	}, {
		key: 'getLogFile',
		value: function getLogFile() {
			return this._logFile;
		}
	}, {
		key: 'setLogName',
		value: function setLogName(name) {
			this._logName = '' + name;
			return this;
		}
	}, {
		key: 'getLogName',
		value: function getLogName() {
			return this._logName;
		}
	}, {
		key: 'setPort',
		value: function setPort(port) {
			port = parseInt(port);

			if (isNaN(port) || port < 0 || port > 65535) {
				throw new Error('Invalid port number.');
			}

			this._port = port;
			return this;
		}
	}, {
		key: 'getPort',
		value: function getPort() {
			return this._port;
		}
	}, {
		key: 'setPayloadMaxSize',
		value: function setPayloadMaxSize(size) {
			this._payloadMaxSize = '' + size;
			return this;
		}
	}, {
		key: 'getPayloadMaxSize',
		value: function getPayloadMaxSize() {
			return this._payloadMaxSize;
		}
	}, {
		key: 'setAuth',
		value: function setAuth(auth) {
			if ((typeof auth === 'undefined' ? 'undefined' : _typeof(auth)) !== 'object' || typeof auth.method !== 'function') {
				throw new Error('Invalid auth config object.');
			}

			this._auth = auth;
			return this;
		}
	}, {
		key: 'getAuth',
		value: function getAuth() {
			return this._auth || {};
		}
	}, {
		key: 'setMetadata',
		value: function setMetadata(metadata) {
			if ((typeof metadata === 'undefined' ? 'undefined' : _typeof(metadata)) !== 'object' || typeof metadata.method !== 'function') {
				throw new Error('Invalid metadata config object.');
			}

			this._metadata = metadata;
			return this;
		}
	}, {
		key: 'getMetadata',
		value: function getMetadata() {
			return this._metadata || {};
		}
	}, {
		key: 'setStorage',
		value: function setStorage(storage) {
			if ((typeof storage === 'undefined' ? 'undefined' : _typeof(storage)) !== 'object' || typeof storage.method !== 'function') {
				throw new Error('Invalid storage config object.');
			}

			this._storage = storage;
			return this;
		}
	}, {
		key: 'getStorage',
		value: function getStorage() {
			return this._storage || {};
		}
	}, {
		key: 'start',
		value: function start() {
			var app = this._app = (0, _connect2.default)();
			var log = this._logger = this._createLogger();

			// Exception handling
			process.on('uncaughtException', function (err) {
				log.error({ type: 'uncaught-exception', stack: err.stack }, '' + err);
			});

			// Add timestamp and logger to request
			app.use(function (req, res, next) {
				req.start = new Date();
				req.log = log;
				next();
			});

			app.use((0, _verifyRequest2.default)());
			app.use((0, _parsePayload2.default)());
			app.use((0, _createSensorData2.default)());
			app.use((0, _authenticate2.default)(this.getAuth()));
			app.use((0, _setMetadata2.default)(this.getMetadata()));
			app.use((0, _concava.convert)());
			app.use((0, _concava.calibrate)());
			app.use((0, _concava.validate)());
			app.use((0, _debug2.default)(this.getDebugMode()));
			app.use((0, _store2.default)(this.getStorage()));
			app.use((0, _errorHandler2.default)(this.getAuth().byToken));

			// End request
			app.use(function (req, res, next) {
				res.end();
				next();
			});

			// Start server
			var port = this.getPort();
			_http2.default.createServer(app).listen(port);
			log.info('Listening on port ' + port + '.');
		}
	}, {
		key: '_createLogger',
		value: function _createLogger() {
			return _bunyan2.default.createLogger({
				name: this.getLogName(),
				streams: [{ level: 'warn', stream: process.stdout }, { level: this.getDebugMode() ? 'debug' : 'info', path: this.getLogPath() }]
			});
		}
	}]);

	return Server;
}();

exports.default = Server;